<?php

// $Id: droptor_feed.inc,v 1.1.2.1 2010/04/14 21:15:03 urbanjustin Exp $ 

/*
	The Droptor bridge module	
	Droptor.com - Drupal site organization, monitoring, management and best practices
	
	For more information, see:
	- http://www.droptor.com
	- http://drupal.org/project/droptor
*/

function droptor_feed () {

	/* 
		We only show the feed if the hash key is passed in and if the request comes from monrupal.com 
		(headers can be spoofed, so we use the hash keys too). It's not paranoia if they're really after you.
	*/
	
	// get hash, domain
	$correct_hash = droptor_check_hash();
	$correct_host = droptor_check_domain();
	$correct_security = droptor_check_connection_security();

	if( $correct_hash && $correct_host && $correct_security ) {
		$data = array();
		$data['droptor-data-feed']['droptor_feed_version'] = '1.1';
		$data['droptor-data-feed']['monitor_text_match'] = variable_get('droptor_monitor_text_match', null);
		$data['droptor-data-feed']['secure_pages_module_enabled'] = droptor_securepages_module_enabled();
		$data['droptor-data-feed']['ssl_login'] = droptor_ssl_login();
		$data['droptor-data-feed']['ssl_enabled'] = droptor_ssl_enabled();
		$data['droptor-data-feed']['settings_permissions_read_only'] = droptor_settings_permissions_read_only();
		$data['droptor-data-feed']['devel_enabled'] = droptor_devel_module_enabled();
		$data['droptor-data-feed']['extra_files_removed'] = droptor_extra_files_removed();
		$data['droptor-data-feed']['page_cache_enabled'] = droptor_caching_enabled();
		$data['droptor-data-feed']['js_compression_enabled'] = droptor_js_compression_enabled();
		$data['droptor-data-feed']['css_compression_enabled'] = droptor_css_compression_enabled();
		$data['droptor-data-feed']['page_compression_enabled'] = droptor_page_compression_enabled();
		$data['droptor-data-feed']['theme_registry_rebuild_enabled'] = droptor_theme_registry_rebuild_enabled();
		$data['droptor-data-feed']['last_cron'] = droptor_last_cron();
		$data['droptor-data-feed']['node_statistics'] = droptor_node_statistics();
		$data['droptor-data-feed']['user_statistics'] = droptor_user_statistics();
		$data['droptor-data-feed']['statistics'] = droptor_statistics();
		$data['droptor-data-feed']['last_nodes'] = droptor_last_nodes();
		$data['droptor-data-feed']['last_users'] = droptor_last_users();
		$data['droptor-data-feed']['watchdog_size'] = droptor_watchdog_size();
		$data['droptor-data-feed']['site_name'] = variable_get('site_name','{unknown}');
		$data['droptor-data-feed']['modules'] = droptor_modules();
		$data['droptor-data-feed']['module_update_data'] = morupal_module_update_data();
		$data['droptor-data-feed']['variables_data'] = droptor_variables_data();
		$data['droptor-data-feed']['recent_pages_not_found'] = droptor_recent_pages_not_found();
		$data['droptor-data-feed']['error_level'] = droptor_error_level();
		$data['droptor-data-feed']['last_logins'] = droptor_last_logins();
		$data['droptor-data-feed']['feed_over_ssl'] = variable_get('droptor_ssl_feed',0);
	
		//droptor_pr($data);
		echo serialize($data);
	}
	else if(!$correct_hash)
		echo t('Invalid hash');
	else if(!$correct_host)
		echo t('Invalid host ('.$_SERVER['REMOTE_ADDR'].')');
	else if(!$correct_security)
		echo t('Invalid security');
	
	exit; // http://stackoverflow.com/questions/2102688/how-do-you-replace-the-page-tpl-php-that-loads-from-a-module-in-drupal
}

// failed logins in the last hour
function droptor_last_logins() {
	$query = 'SELECT message, variables, timestamp FROM {watchdog} WHERE message LIKE "login attempt failed%" AND now()-timestamp > 60*60 ORDER BY timestamp DESC';
	$results = db_query($query);
	$last_logins = array();
	
	while($result = db_fetch_array($results)) {
		$variables = unserialize($result['variables']);
		$last_logins['failed'][$result['timestamp']] = $variables['%user'];
	}
	return $last_logins;
}

function droptor_error_level() {
	return variable_get('error_level',1);
}

function droptor_variables_data() {
	$variables = array();
	$results = db_query('SELECT name, value FROM {variable} ORDER BY name');
	
	while($result = db_fetch_array($results)) {
		$variables[$result['name']] = unserialize($result['value']);
	}
	return serialize($variables);
}

function droptor_recent_pages_not_found() {
	$data = array();
	$row = 0;
	
	$results = db_query('SELECT message,hostname,referer,timestamp FROM {watchdog} WHERE type = "page not found" AND message NOT IN ("UPGRADE.txt","MAINTAINERS.txt","README.txt","INSTALL.pgsql.txt","INSTALL.txt","LICENSE.txt","INSTALL.mysql.txt","COPYRIGHT.txt","CHANGELOG.txt") ORDER BY timestamp DESC LIMIT 10');
	
	while($result = db_fetch_array($results)) {
		$data[$row]['message'] = $result['message'];
		$data[$row]['hostname'] = $result['hostname'];
		$data[$row]['referer'] = $result['referer'];
		$data[$row]['timestamp'] = $result['timestamp'];
		
		$row++;
	}
	
	return $data;
}

function droptor_watchdog_size() {
	return db_result(db_query('SELECT COUNT(*) FROM {watchdog}'));
}

function droptor_node_statistics() {
	$results = db_query("SELECT COUNT(*) AS total,DATE_FORMAT(FROM_UNIXTIME(created),'%m/%%d/%Y') AS dt FROM node GROUP BY DATE_FORMAT(FROM_UNIXTIME(created),'%m/%%d/%Y') ORDER BY created DESC LIMIT 30");
	$data = array();
	while($result = db_fetch_array($results)) {
		$data[$result['dt']] = $result['total'];
	}
	
	return $data;
}

function droptor_user_statistics() {
	$results = db_query("SELECT COUNT(*) AS total,DATE_FORMAT(FROM_UNIXTIME(created),'%m/%%d/%Y') AS dt FROM users GROUP BY DATE_FORMAT(FROM_UNIXTIME(created),'%m/%%d/%Y') ORDER BY created DESC LIMIT 30");
	$data = array();
	while($result = db_fetch_array($results)) {
		$data[$result['dt']] = $result['total'];
	}
	
	return $data;
}

function droptor_last_nodes() {
	$last_five_nodes = array();
	$query = db_query('SELECT title, nid, created FROM {node} ORDER BY created DESC LIMIT 15');

	$count = 0;
	while($data = db_fetch_array($query)) {
		$last_five_nodes[$count]['url'] = drupal_get_path_alias('node/'.$data['nid']);
		$last_five_nodes[$count]['title'] = $data['title'];
		$last_five_nodes[$count]['created'] = $data['created'];
		$count++;
	}
	
	return $last_five_nodes;
	
}

function droptor_last_users() {
	$last_five_users = array();
	$query = db_query('SELECT uid, mail, name, created FROM {users} ORDER BY created DESC LIMIT 15');

	$count = 0;
	while($data = db_fetch_array($query)) {
		$last_five_users[$count]['uid'] = $data['uid'];
		$last_five_users[$count]['name'] = $data['name'];
		$last_five_users[$count]['email'] = $data['mail'];
		$last_five_users[$count]['created'] = $data['created'];
		$count++;
	}
	
	return $last_five_users;
}

// nodes and users in the last 24 hours, 7 days and 30 days; nodes_in_the_last_24_hours  	
function droptor_statistics() {
	$stats = array();
	
	// node and user creations over the last month
	$stats = droptor_node_and_user_time_stats();
	$stats = array_merge($stats,droptor_node_and_user_totals());
	
	return $stats;
}

function droptor_last_cron() {
	$last_cron = variable_get('cron_last',null);
	
	return $last_cron ? date('Y-m-d H:i:s',$last_cron) : false;
}

function droptor_theme_registry_rebuild_enabled() {
	return variable_get('devel_rebuild_theme_registry',false) == 1 ? 1 : 0;
}

function droptor_js_compression_enabled() {
	return variable_get('preprocess_js',false) == 1 ? 1 : 0;
}

function droptor_page_compression_enabled() {
	return variable_get('page_compression',false) == 1 ? 1 : 0;
}

function droptor_css_compression_enabled() {
	return variable_get('preprocess_css',false) == 1 ? 1 : 0;
}

function droptor_caching_enabled() {
	return variable_get('cache',false) == 1 ? 1 : 0;
}

function morupal_module_update_data() {
	$module_meta_data = update_get_available(TRUE);
	return serialize(update_calculate_project_data($module_meta_data));
}

function droptor_modules() {
	$query_result = db_query('SELECT name, status, info, type FROM {system} ORDER BY status DESC,name');
	$modules = array();
		
	while($data = db_fetch_object($query_result)) {
		$info = unserialize($data->info);
		$modules[$data->name]['version'] = $info['version'];
		$modules[$data->name]['status'] = $data->status;
		$modules[$data->name]['name'] = $info['name'];
		$modules[$data->name]['type'] = $data->type;
	}
	
	return serialize($modules);
}

function droptor_extra_files_removed() {
	$clean_codebase = true;
	$url = url(NULL, array('absolute' => TRUE));
	$files_to_remove = array('CHANGELOG.txt','COPYRIGHT.txt','INSTALL.mysql.txt','INSTALL.pgsql.txt','INSTALL.txt','LICENSE.txt',
		'MAINTAINERS.txt','README.txt','UPGRADE.txt');
	
	foreach ($files_to_remove as $file) {
		if(droptor_fast_url_check($url.$file))
			$clean_codebase = false;
	}
	
	return $clean_codebase ? 1 : 0;
}

function droptor_devel_module_enabled() {
	return droptor_is_module_installed_and_enabled('devel');
}

function droptor_settings_permissions_read_only() {
	$settings_permissions_read_only = true;
	$writes = array('2','3','6','7'); // http://en.wikipedia.org/wiki/File_system_permissions
	$settings_file = './'. conf_path(FALSE, TRUE) .'/'. $prefix .'settings.php';
	$permissions = substr(sprintf('%o', fileperms($settings_file)), -4);	
	
	foreach($writes as $bit) {
		if(strpos($permissions,$bit)) {
			$settings_permissions_read_only = false;			
			break;
		}
	}
	
	return $settings_permissions_read_only;
	
}

function droptor_securepages_module_enabled() {
	// secure pages module is installed and enabled
	return droptor_is_module_installed_and_enabled('securepages') && variable_get('securepages_enable',false) ? true : false;
}

function droptor_ssl_login() {
	$login_safe = 0;
	$securepages_login_query = db_query('SELECT value FROM variable WHERE name = "%s"','securepages_pages');
	while($securepages_login_results = db_fetch_object($securepages_login_query)) {
		$pages = $securepages_login_results->value;
		if(strpos($pages,'user/') > 0 || strpos($pages,'user*') > 0)
			$login_safe = 1; // to do - check the ignore pages field			
	}
	
	return $login_safe;
}

/* MENU */
/* MENU */
/* MENU */

function droptor_node_and_user_time_stats() {
	$thirty_days_ago_in_seconds = time() - (30*24*60*60);
	$seven_days_ago_in_seconds = time() - (7*24*60*60);
	$twenty_four_hours_ago_in_seconds = time() - (24*60*60);

	$stats = array();
	$stats['nodes_in_the_last_30_days'] = 0;
	$stats['nodes_in_the_last_7_days'] = 0;
	$stats['nodes_in_the_last_24_hours'] = 0;
	$stats['users_in_the_last_30_days'] = 0;
	$stats['users_in_the_last_7_days'] = 0;
	$stats['users_in_the_last_24_hours'] = 0;
	
	$nodes_result = db_query('SELECT created FROM node WHERE created > %d',$thirty_days_ago_in_seconds);
	 
	while($node = db_fetch_object($nodes_result)) {
		if($node->created > $twenty_four_hours_ago_in_seconds)
			$stats['nodes_in_the_last_24_hours']++;
			
		if($node->created > $seven_days_ago_in_seconds)
			$stats['nodes_in_the_last_7_days']++;
			
		if($node->created > $thirty_days_ago_in_seconds)
			$stats['nodes_in_the_last_30_days']++;
	}
	
	$users_result = db_query('SELECT created FROM users WHERE created > %d',$thirty_days_ago_in_seconds);
	 
	while($user = db_fetch_object($users_result)) {
		if($user->created > $twenty_four_hours_ago_in_seconds)
			$stats['users_in_the_last_24_hours']++;
		
		if($user->created > $seven_days_ago_in_seconds)
			$stats['users_in_the_last_7_days']++;
		
		if($user->created > $thirty_days_ago_in_seconds)
			$stats['users_in_the_last_30_days']++;
	}
	
	return $stats;
}

/* INTERNALS */
/* INTERNALS */
/* INTERNALS */

function droptor_is_module_installed_and_enabled($name) {
	$module_query = db_query('SELECT count(*) as module_installed from system WHERE name = "%s" and status = 1',$name);

	while($module_results = db_fetch_object($module_query)) {
		return $module_results->module_installed;
	}
}

// cause drupal_http_request is wicked slow compared to straight curl
function droptor_fast_url_check($url) {
	$valid = false;
	
	if(in_array('curl', get_loaded_extensions())) {
		$curl = curl_init();	
		curl_setopt($curl, CURLOPT_URL, $url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($curl, CURLOPT_HEADER, false);
		curl_exec($curl);
		$result = curl_getinfo($curl);
		curl_close($curl);
		$valid = $result['http_code'] == '200' ? true : false;
	} else {
		$result = drupal_http_request($url);
		$valid = $result->code == 200 ? true : false;
	}	
	
	return $valid;
}

function droptor_node_and_user_totals() {
	$stats = array();
		
	// total users
	$total_users_query = db_query('SELECT COUNT(*) as total_users FROM {users}');
	$total_users_result = db_fetch_array($total_users_query);
	$stats = is_array($total_users_result) && count($total_users_result) == 1 ? array_merge($stats,$total_users_result) : $stats;

	// total nodes
	$total_nodes_query = db_query('SELECT COUNT(*) as total_nodes, nt.name as type FROM {node} n JOIN {node_type} nt ON n.type = nt.type GROUP BY type ORDER BY nt.name');
	$total_nodes = 0;
	$nodes_by_type = array();
	//$total_nodes_result = db_fetch_array($total_nodes_query);
	//$stats = is_array($total_nodes_result) && count($total_nodes_result) == 1 ? array_merge($stats,$total_nodes_result) : $stats;
	
	while($node_type_data = db_fetch_array($total_nodes_query)) {
		$nodes_by_type[$node_type_data['type']] = $node_type_data['total_nodes'];
		$total_nodes += $node_type_data['total_nodes'];
	}
	
	$stats['total_nodes'] = $total_nodes;
	$stats['total_nodes_by_type'] = $nodes_by_type;
		
	return $stats;
}


?>